package com.diagbot.aggregate;

import com.diagbot.client.PushNewServiceClient;
import com.diagbot.dto.DictionaryDTO;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.facade.KlDictionaryInfoFacade;
import com.diagbot.facade.KlLexiconFacade;
import com.diagbot.facade.KlRulePlanFacade;
import com.diagbot.process.PushProcess;
import com.diagbot.util.ListUtil;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description: 获取字典类型聚合
 * @author: zhoutg
 * @time: 2019/10/16 13:37
 */
@Component
@Slf4j
public class DictionaryAggregate {

    @Autowired
    PushProcess pushProcess;
    @Autowired
    PushNewServiceClient pushNewServiceClient;
    @Autowired
    KlDictionaryInfoFacade klDictionaryInfoFacade;
    @Autowired
    KlRulePlanFacade klRulePlanFacade;
    @Autowired
    KlLexiconFacade klLexiconFacade;

    @DataProvider("getDictionary")
    public DictionaryDTO getDictionary(
            @DataConsumer("getDic") List<DictionaryInfoDTO> dicList,
            @DataConsumer("getLexicon") List<DictionaryInfoDTO> lexiconList,
            @DataConsumer("getPlan") List<DictionaryInfoDTO> planList) {

        DictionaryDTO res = new DictionaryDTO();
        List<DictionaryInfoDTO> list = res.getList();

        if (ListUtil.isNotEmpty(dicList)) {
            list.addAll(dicList);
        }
        if (ListUtil.isNotEmpty(lexiconList)) {
            list.addAll(lexiconList);
        }
        if (ListUtil.isNotEmpty(planList)) {
            list.addAll(planList);
        }
        return res;
    }

    /**
     * 获取字典数据
     *
     * @return
     */
    @DataProvider("getDic")
    public List<DictionaryInfoDTO> getDic() {
        return klDictionaryInfoFacade.getListBackCommon();
    }

    /**
     * 获取lexicon字典数据
     *
     * @return
     */
    @DataProvider("getLexicon")
    public List<DictionaryInfoDTO> getLexicon() {
        return klLexiconFacade.getLexiconDic();
    }

    /**
     * 获取rulePlan字典数据
     *
     * @return
     */
    @DataProvider("getPlan")
    public List<DictionaryInfoDTO> getPlan() {
        return klRulePlanFacade.getPlanDicFac();
    }
}
