package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.*;
import com.diagbot.entity.KlRule;
import com.diagbot.entity.KlRuleBase;
import com.diagbot.entity.KlRuleCondition;
import com.diagbot.enums.CommonEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.RedisEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.KlRuleServiceImpl;
import com.diagbot.util.*;
import com.diagbot.vo.*;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.facade.DataFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.*;

/**
 * @Description:
 * @author: gaodm
 * @time: 2021/2/3 16:44
 */
@Component
public class KlRuleFacade extends KlRuleServiceImpl {
    @Autowired
    KlRuleConditionFacade klRuleConditionFacade;
    @Autowired
    KlRuleBaseFacade klRuleBaseFacade;
    @Autowired
    private RedisUtil redisUtil;

    /**
     * @param klRuleInfoVO
     * @return
     */
    public IPage<KlRuleInfoDTO> getKlRuleInfoPage(KlRuleInfoVO klRuleInfoVO) {
        return this.getKlRuleInfoPages(klRuleInfoVO);
    }

    public KlRuleByIdParDTO getByIdRuleInfos(KlRuleByIdVO klRuleByIdVO) {
        KlRuleByIdParDTO klRuleByIdPar = new KlRuleByIdParDTO();
        List<KlRuleByIdSubDTO> klRuleByIdSubLsit = new ArrayList<>();
        List<KlRuleByIdDTO> byIdRuleInfo = this.getByIdRuleInfo(klRuleByIdVO);
        if (ListUtil.isNotEmpty(byIdRuleInfo)) {
            klRuleByIdPar.setParId(byIdRuleInfo.get(0).getParId());
            klRuleByIdPar.setParDescription(byIdRuleInfo.get(0).getParDescription());
            klRuleByIdPar.setParRuleType(byIdRuleInfo.get(0).getParRuleType());
            klRuleByIdPar.setParConceptId(byIdRuleInfo.get(0).getParConceptId());
            klRuleByIdPar.setParlibName(byIdRuleInfo.get(0).getParlibName());
            klRuleByIdPar.setParHasSub(byIdRuleInfo.get(0).getParHasSub());//是否有子条件（0：无，1：有）
            klRuleByIdPar.setParMsg(byIdRuleInfo.get(0).getParMsg());//附加信息
            klRuleByIdPar.setParStatus(byIdRuleInfo.get(0).getParStatus());//启用状态（0：禁用，1：启用）
            klRuleByIdPar.setParLenName(byIdRuleInfo.get(0).getParLenName());
            klRuleByIdPar.setParLenCode(byIdRuleInfo.get(0).getParLenCode());
            for (KlRuleByIdDTO data : byIdRuleInfo) {
                if (null != data.getSubDescription()) {
                    KlRuleByIdSubDTO klRuleByIdSub = new KlRuleByIdSubDTO();
                    BeanUtil.copyProperties(data, klRuleByIdSub);
                    klRuleByIdSubLsit.add(klRuleByIdSub);
                }
            }
        }
        klRuleByIdPar.setKlRuleByIdSub(klRuleByIdSubLsit);
        return klRuleByIdPar;
    }

    public Boolean saveRuleInfos(KlRuleInfoSaveVO klRuleInfoSaveVO) {
        Date now = DateUtil.now();
        boolean res = false;
/*        // 校验名称是否相同
        int count = this.count(new QueryWrapper<KlRule>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("description", klRuleInfoSaveVO.getParDescription())
                .ne("id", klRuleInfoSaveVO.getParId() == null ? -1 : klRuleInfoSaveVO.getParId()));
        if (count > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该规则名称已存在");
        }*/
        //校验数据是否还在
        if (null != klRuleInfoSaveVO.getParId()) {
            int sum = this.count(new QueryWrapper<KlRule>().eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", klRuleInfoSaveVO.getParId()));
            if (sum == 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该数据已不存在！");
            }
            UpdateWrapper<KlRule> klRuleUpdate = new UpdateWrapper<>();
            klRuleUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", klRuleInfoSaveVO.getParId())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParDescription()), "description", klRuleInfoSaveVO.getParDescription())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParConceptId().toString()), "concept_id", klRuleInfoSaveVO.getParConceptId())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParRuleType().toString()), "rule_type", klRuleInfoSaveVO.getParRuleType())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParHasSub().toString()), "has_sub_cond", klRuleInfoSaveVO.getParHasSub())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParStatus().toString()), "status", klRuleInfoSaveVO.getParStatus())
                    .set(StringUtil.isNotBlank(klRuleInfoSaveVO.getParMsg()), "msg", klRuleInfoSaveVO.getParMsg())
                    .set("modifier", klRuleInfoSaveVO.getModifier())
                    .set("gmt_modified", now);
            res = this.update(klRuleUpdate);

            //先删除原有的详情;
            res = clearSub(klRuleInfoSaveVO.getParId());
            if (ListUtil.isNotEmpty(klRuleInfoSaveVO.getKlRuleInfoSaveSub())) {
                //再重新传入的详情保存
                res = saveCommon(klRuleInfoSaveVO);
            }
        } else {
            //先保存主表rule
            KlRule klRule = new KlRule();
            klRule.setConceptId(klRuleInfoSaveVO.getParConceptId());
            klRule.setCreator(klRuleInfoSaveVO.getModifier());
            klRule.setModifier(klRuleInfoSaveVO.getModifier());
            klRule.setDescription(klRuleInfoSaveVO.getParDescription());
            klRule.setHasSubCond(klRuleInfoSaveVO.getParHasSub());
            klRule.setRuleType(klRuleInfoSaveVO.getParRuleType());
            klRule.setMsg(klRuleInfoSaveVO.getParMsg());
            klRule.setStatus(klRuleInfoSaveVO.getParStatus());
            klRule.setGmtCreate(now);
            klRule.setGmtModified(now);
            res = this.save(klRule);
            if (res) {
                //再保存附表
                klRuleInfoSaveVO.setParId(klRule.getId());
                res = saveCommon(klRuleInfoSaveVO);
            }
        }

        return res;
    }


    public Boolean clearRuleInfos(KlRuleInfoClearVO klRuleInfoClearVO) {
        boolean res = false;
        res = clearSub(klRuleInfoClearVO.getId());
        //最后删除主表rule
        res = this.removeById(klRuleInfoClearVO.getId());
        return res;
    }

    /**
     * 删除明细表
     */
    public boolean clearSub(Long id) {
        boolean res = false;
        QueryWrapper<KlRuleCondition> klRuleConditionQuery = new QueryWrapper<>();
        klRuleConditionQuery.eq("is_deleted", IsDeleteEnum.N.getKey()).eq("rule_id", id);
        List<KlRuleCondition> ruleConditionList = klRuleConditionFacade.list(klRuleConditionQuery);
        List<Long> ruleBaseId = new ArrayList<>();
        if (ListUtil.isNotEmpty(ruleConditionList)) {
            for (KlRuleCondition data : ruleConditionList) {
                ruleBaseId.add(data.getRuleBaseId());
            }
        }
        if (ListUtil.isNotEmpty(ruleBaseId)) {
            QueryWrapper<KlRuleBase> klRuleBaseQuery = new QueryWrapper<>();
            klRuleBaseQuery.eq("is_deleted", IsDeleteEnum.N.getKey()).in("id", ruleBaseId);
            //删除kl_rule_base表数据
            res = klRuleBaseFacade.remove(klRuleBaseQuery);

        }
        //删除kl_rule_condition表的数据
        res = klRuleConditionFacade.remove(klRuleConditionQuery);
        return res;
    }

    /**
     * 保存数据
     */
    public boolean saveCommon(KlRuleInfoSaveVO klRuleInfoSaveVO) {
        Date now = DateUtil.now();
        boolean res = false;
        List<KlRuleInfoSaveSubVO> klRuleInfoSaveSub = klRuleInfoSaveVO.getKlRuleInfoSaveSub();
        if (ListUtil.isNotEmpty(klRuleInfoSaveSub)) {
            //先保存数据到kl_rule_base表里
            List<KlRuleBase> klRuleBaseList = new ArrayList<>();
            for (KlRuleInfoSaveSubVO list : klRuleInfoSaveSub) {
                KlRuleCondition klRuleCondition = new KlRuleCondition();
                KlRuleBase klRuleBase = new KlRuleBase();
                klRuleBase.setGmtModified(now);
                klRuleBase.setModifier(klRuleInfoSaveVO.getModifier());
                klRuleBase.setConceptId(list.getSubConceptId());
                klRuleBase.setType(list.getSubType());
                klRuleBase.setDescription(list.getSubDescription());
                klRuleBase.setMaxOperator(list.getSubMaxOperator());
                klRuleBase.setMaxValue(list.getSubMaxValue());
                klRuleBase.setMaxUnit(list.getSubMaxUnit());
                klRuleBase.setMinOperator(list.getSubMinOperator());
                klRuleBase.setMinValue(list.getSubMinValue());
                klRuleBase.setMinUnit(list.getSubMinUnit());
                klRuleBase.setStatus(1);
                klRuleBase.setEqValue(list.getSubEqValue());
                klRuleBase.setEqOperator(list.getSubEqOperator());
                klRuleBase.setEqUnit(list.getSubEqUnit());
                res = klRuleBaseFacade.save(klRuleBase);
                //再把数据保存在kl_rule_condition
                if (res) {
                    klRuleCondition.setGmtCreate(now);
                    klRuleCondition.setModifier(klRuleInfoSaveVO.getModifier());
                    klRuleCondition.setGroupType(list.getGroupType());
                    klRuleCondition.setRuleBaseId(klRuleBase.getId());
                    klRuleCondition.setRuleId(klRuleInfoSaveVO.getParId());
                    klRuleCondition.setMsg(list.getSubMsg());
                    res = klRuleConditionFacade.save(klRuleCondition);
                }

            }
        }

        return res;
    }

    public Boolean startOrDisableRule(KlRuleSatartOrdisaVO klRuleSatartOrdisaVO, int start) {
        boolean res = false;
        Date now = DateUtil.now();
        int sum = this.count(new QueryWrapper<KlRule>().eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klRuleSatartOrdisaVO.getId()));
        if (sum == 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该数据已不存在！");
        }
        UpdateWrapper<KlRule> klRuleUpdate = new UpdateWrapper<>();
        klRuleUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klRuleSatartOrdisaVO.getId())
                .set("status", start)
                .set("modifier", klRuleSatartOrdisaVO.getModifier())
                .set("gmt_modified", now);
        return this.update(klRuleUpdate);
    }

    /**
     * 开单合理性导出
     *
     * @return
     */
    public void exportIndicationFac(HttpServletResponse response) {
        List<IndicationExportDTO> indicationExportDTOS = exportIndication();
        ExcelUtils.exportExcel(indicationExportDTOS, null, "开单合理性", IndicationExportDTO.class, "indication.xlsx",
                response, 12.8f);
    }


    /**
     * 是否集合类型
     *
     * @param type
     * @return
     */
    public Boolean isGatherType(Integer type) {
        if (type == null) {
            return false;
        }
        return (type >= 308 && type <= 329) || (type >= 301 && type <= 304);
    }

    /**
     * 查询所有有效的规则
     *
     * @param ruleQueryVO
     * @return
     */
    public IPage<RuleQueryDTO> getRulePageFac(RuleQueryVO ruleQueryVO) {
        if (StringUtil.isNotBlank(ruleQueryVO.getBaseLibName()) || !IntegerUtil.isNull(ruleQueryVO.getBaseLibType())) {
            try {
                //取得所有基础规则条目的上级规则的ID
                Map<String, Object> invokeParams = new HashMap<>();
                RuleBaseQueryVO ruleBaseQueryVO = new RuleBaseQueryVO();
                ruleBaseQueryVO.setBaseLibType(ruleQueryVO.getBaseLibType());
                ruleBaseQueryVO.setBaseLibName(ruleQueryVO.getBaseLibName());
                invokeParams.put("ruleBaseQueryVO",ruleBaseQueryVO);
                List<Long> ruleIds = DataFacade.get("getAllRuleIds", invokeParams, List.class);
                ruleQueryVO.setRuleIds(ruleIds);
            } catch (Exception e) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "查询所有有效规则失败" + e.getMessage());
            }
        }
        return this.getRulePage(ruleQueryVO);
    }

    /**
     * 根据录入内容查询有效规则的明细
     *
     * @param ruleQueryKeyVO
     * @return
     */
    public RuleDTO getRuleDetail(RuleQueryKeyVO ruleQueryKeyVO) {
        String key = RedisEnum.allRule.getName() + ruleQueryKeyVO.getLibType() + "_" + ruleQueryKeyVO.getLibName() + "_" + ruleQueryKeyVO.getRuleType();
        RuleDTO ruleDTO = redisUtil.get(key);

        // 规则名称入参，转大写
        String descriptionVO = toUpperCase(ruleQueryKeyVO.getDescription());
        // 基础术语名称入参，转大写
        String baseLibNameVO = toUpperCase(ruleQueryKeyVO.getBaseLibName());

        // 基础术语类型入参
        Integer baseLibTypeVO = ruleQueryKeyVO.getBaseLibType();

        // 有条件才去遍历筛选
        if (StringUtil.isNotEmpty(descriptionVO) || StringUtil.isNotEmpty(baseLibNameVO) || baseLibTypeVO != null) {
            List<RuleConditionDTO> ruleConditionDTOList = ruleDTO.getRuleConditionDTOList();
            Iterator<RuleConditionDTO> iterator = ruleConditionDTOList.iterator();
            while (iterator.hasNext()) {
                RuleConditionDTO ruleConditionDTO = iterator.next();
                // 过滤规则名称
                if (StringUtil.isNotBlank(descriptionVO)) {
                    if (!toUpperCase(ruleConditionDTO.getDescription()).contains(descriptionVO)) {
                        iterator.remove();
                        continue;
                    }
                }

                // 过滤基础术语类型和名称
                if ((baseLibTypeVO != null || StringUtil.isNotEmpty(ruleQueryKeyVO.getBaseLibName()))) {
                    if (ListUtil.isEmpty(ruleConditionDTO.getRuleBaseDTOList())) {
                        iterator.remove();
                        continue;
                    }
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    boolean hasFlag = false; // 是否匹配
                    for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
                        Integer baseLibType = ruleBaseDTO.getBaseLibType();
                        String baseLibName = toUpperCase(ruleBaseDTO.getBaseLibName());

                        if (StringUtil.isNotBlank(baseLibNameVO) && baseLibTypeVO != null) {
                            if (baseLibName.contains(baseLibNameVO) && baseLibTypeVO.equals(baseLibType)) {
                                hasFlag = true;
                                break;
                            }
                        } else if (StringUtil.isNotBlank(baseLibNameVO)) {
                            if (baseLibName.contains(baseLibNameVO)) {
                                hasFlag = true;
                                break;
                            }
                        } else if (baseLibTypeVO != null) {
                            if (baseLibTypeVO.equals(baseLibType)) {
                                hasFlag = true;
                                break;
                            }
                        }
                    }
                    if (!hasFlag) {
                        iterator.remove();
                        continue;
                    }
                }
            }
        }
        return ruleDTO;
    }

    /**
     * 转大写
     *
     * @param name
     * @return
     */
    public String toUpperCase(String name) {
        if (StringUtil.isNotBlank(name)) {
            return name.toUpperCase();
        }
        return "";
    }
}
